# -*- coding: utf-8 -*-
from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.cmsutils import get_abs_url_from_slug
from django.core.mail import EmailMultiAlternatives
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.template.loader import get_template
from django.template import Context

from acomea.simulatoreptf.cms.models import SimulatorePTFSettings, SharingPTFSettings
from acomea.simulatoreptf.simulazione import Simulazione
from .forms import SendMail


class SimulatorePTFPlugin(CMSPluginBase):
    model = SimulatorePTFSettings
    name = _("Simulatore PTF Plugin")
    module = _("Simulatore PTF")
    admin_preview = False
    render_template = "bootstrap3/simulatore_ptf/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        share_url = request.session.get("simulatore_ptf_share_url", request.build_absolute_uri())
        if user.is_authenticated():
            form_class = SendMail(initial={'email': user.email})
        else:
            form_class = SendMail
        if instance.typology == 'optional':
            simulazione = Simulazione(
                request,
                url=settings.SIMULATORE_PTF_OPTIONAL_URL,
                optional=instance.optional_type
            )
        else:
            simulazione = Simulazione(
                request,
                url=settings.SIMULATORE_PTF_PLUGIN_URL,
            )
        self.render_template = instance.template_name

        if request.method == 'POST':
            simulatore_form = SendMail(request.POST)
            if simulatore_form.is_valid():
                email = request.POST.get('email')
                try:
                    validate_email(email)
                    plaintext = get_template("bootstrap3/simulatore_ptf/cms_plugins/email/memo.html")
                    htmly = get_template("bootstrap3/simulatore_ptf/cms_plugins/email/memo.html")
                    d = Context({
                        'username': request.user,
                        'link': share_url
                    })
                    subject, from_email, to = \
                        'acomea memo simulatorePTF', \
                        'memo@acomea.it', email
                    text_content = plaintext.render(d)
                    html_content = htmly.render(d)
                    msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
                    msg.attach_alternative(html_content, "text/html")
                    msg.send()
                    if 'simulatore_ptf_share_url' in request.session:
                        del request.session['simulatore_ptf_share_url']
                    raise ForceResponse(HttpResponseRedirect(share_url))
                except ValidationError as e:
                    context.update({'err': u"L'email inserita non è valida"})

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'user': user,
            'simulazione': simulazione,
            'form': form_class,
        })
        return context


class SharingPTFPlugin(CMSPluginBase):
    model = SharingPTFSettings
    name = _("Sharing PTF Plugin")
    module = _("Simulatore PTF")
    admin_preview = False
    render_template = "bootstrap3/simulatore_ptf/cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        if request.method == 'GET':
            simulation_id = request.GET.get("session_id", None)
            if simulation_id:
                request.session["simulatore_ptf_sharing_session_id"] = simulation_id.strip()
                request.session["simulatore_ptf_share_url"] = request.build_absolute_uri()
                redirect_to = get_abs_url_from_slug(instance.redirect_to)
                raise ForceResponse(HttpResponseRedirect(redirect_to))
        context.update({
            'object': instance,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(SimulatorePTFPlugin)
plugin_pool.register_plugin(SharingPTFPlugin)
