#! -*- coding: utf-8 -*-
import base64
import hashlib
import time
from django.conf import settings

from jmb.core.utils.functions import zerofill
from acomea.app.soap import OpServizi


class Simulazione(object):
    """
    Class used to call deustechnology API
    """

    def __init__(self, request, url=settings.SIMULATORE_PTF_PLUGIN_URL, optional=None):
        self.request = request
        self.user = request.user
        self.id_rac = self.codice_cliente = self.codice_simulazione = 0
        self.url = url
        self.optional = optional
        self.set_values()

    def get_profilo(self):
        if hasattr(self.user, 'profilo'):
            return self.user.profilo
        return None

    def get_rac(self):
        profilo = self.get_profilo()
        if profilo and hasattr(profilo, 'rac'):
            return profilo.rac
        return None

    def is_anonymous(self):
        return self.user.is_anonymous()

    def is_only_lead(self):
        if self.is_anonymous():
            return False
        profilo = self.get_profilo()
        if profilo and profilo.is_only_lead:
            return True
        return False

    def is_cliente(self):
        if self.is_anonymous():
            return False
        profilo = self.get_profilo()
        if profilo and profilo.is_cliente:
            return True
        return False

    def _set_id_rac(self, id_rac):
        self.id_rac = id_rac

    def _set_codice_cliente(self, codice_cliente):
        self.codice_cliente = codice_cliente

    def _set_codice_simulazione(self, codice_simulazione):
        self.codice_simulazione = codice_simulazione

    def set_values(self):
        # Anonymous user
        if self.is_anonymous():
            pass
        # Only lead user
        if self.is_only_lead():
            rac = self.get_rac()
            if rac:
                self._set_id_rac(rac.id)
        # Cliente user. Clienti with only gestioni has not CodCliFC
        if self.is_cliente():
            rac = self.get_rac()
            self._set_id_rac(rac.id)
            if rac.stato in ('6', '9'):
                codice_fiscale = rac.codice_fiscale
                os = OpServizi()
                cliente = os.get_cliente(codice_fiscale)
                if cliente and 'CodCliFC' in cliente:
                    self._set_codice_cliente(int(cliente['CodCliFC']))
        # Retrieve session_id if available in request.sessionì
        sharing_key = "simulatore_ptf_sharing_session_id"
        if sharing_key in self.request.session and self.request.session[sharing_key]:
            self._set_codice_simulazione(self.request.session[sharing_key])
            del self.request.session[sharing_key]

    def get_token(self):
        timestamp = int(time.time() * 1000)
        zerofilles = "%s|%s|%s" % (
            zerofill(self.id_rac, 8),
            zerofill(self.codice_cliente, 8),
            zerofill(self.codice_simulazione, 32) if not self.codice_simulazione else self.codice_simulazione,
        )
        digest_sha = hashlib.sha1('%s:%s:%s' % (
            zerofilles,
            timestamp,
            settings.SIMULATORE_PTF_SECRET_KEY
        )).hexdigest()
        digest_base64 = base64.b64encode('%s:%s:%s' % (
            zerofilles, timestamp, digest_sha
        ))
        return digest_base64

    def get_url(self):
        url = "{url}?tkn={token}".format(
            url=self.url,
            token=self.get_token()
        )
        if self.optional:
            url += '&type={}'.format(self.optional)
        return url
